@@ -5,11 +5,13 @@
 # later version. See the file COPYING for details.
 
 
-from xpra.util import nonl, csv, typedict
+from xpra.util import nonl, csv
 from xpra.os_util import POSIX, OSX, bytestostr
 from xpra.server.rfb.rfb_const import RFBEncoding, RFB_KEYNAMES
 from xpra.server.rfb.rfb_protocol import RFBProtocol
 from xpra.server.rfb.rfb_source import RFBSource
+from xpra.server import server_features
+from xpra.scripts.config import parse_bool, parse_number
 
 from xpra.log import Logger
 log = Logger("rfb")
@@ -21,6 +23,8 @@ log = Logger("rfb")
 class RFBServer(object):
 
     def __init__(self):
+        self._window_to_id = {}
+        self._rfb_upgrade = 0
         self.readonly = False
         self.rfb_buttons = 0
         self.x11_keycodes_for_keysym = {}
@@ -28,10 +32,20 @@ class RFBServer(object):
             from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
             self.X11Keyboard = X11KeyboardBindings()
 
+    def init(self, opts):
+        if not parse_bool("rfb-upgrade", opts.rfb_upgrade):
+            self._rfb_upgrade = 0
+        else:
+            self._rfb_upgrade = parse_number(int, "rfb-upgrade", opts.rfb_upgrade, 0)
+        log("init(..) rfb-upgrade=%i", self._rfb_upgrade)
+
 
     def _get_rfb_desktop_model(self):
         models = tuple(self._window_to_id.keys())
-        if len(models)!=1:
+        if not models:
+            log.error("RFB: no window models to export, dropping connection")
+            return None
+        elif len(models)!=1:
             log.error("RFB can only handle a single desktop window, found %i", len(self._window_to_id))
             return None
         return models[0]
@@ -105,10 +119,11 @@ class RFBServer(object):
         #shadow servers need to be told to start the refresh timer:
         start_refresh = getattr(self, "start_refresh", None)
         if start_refresh:
-            start_refresh()
+            for wid in tuple(self._window_to_id.values()):
+                start_refresh(wid)
 
     def _process_rfb_PointerEvent(self, _proto, packet):
-        if self.readonly:
+        if not server_features.input_devices or self.readonly:
             return
         buttons, x, y = packet[1:4]
         wid = self._get_rfb_desktop_wid()
@@ -123,7 +138,7 @@ class RFBServer(object):
             self.rfb_buttons = buttons
 
     def _process_rfb_KeyEvent(self, proto, packet):
-        if self.readonly:
+        if not server_features.input_devices or self.readonly:
             return
         source = self._server_sources.get(proto)
         if not source:
@@ -161,6 +176,7 @@ class RFBServer(object):
     def _process_rfb_FramebufferUpdateRequest(self, _proto, packet):
         #pressed, _, _, keycode = packet[1:5]
         inc, x, y, w, h = packet[1:6]
+        log("RFB: FramebufferUpdateRequest inc=%s, geometry=%s", inc, (x, y, w, h))
         if not inc:
             model = self._get_rfb_desktop_model()
             self._damage(model, x, y, w, h)
